﻿#region License Information

/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

#endregion

using System;
using System.Linq;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  public class SymbolicDataAnalysisModelComplexityAnalyzer {
    public static double CalculateComplexity(ISymbolicExpressionTreeNode node) {
      if (node is ConstantTreeNode) return 0;
      if (node is VariableTreeNode) return 1;

      var subtreeComplexities = node.Subtrees.Select(CalculateComplexity);

      double result = 0.0;
      switch (OpCodes.MapSymbolToOpCode(node)) {
        case OpCodes.Add:
        case OpCodes.Sub:
          foreach (var comp in subtreeComplexities) {
            result += comp;
          }
          break;
        case OpCodes.Mul:
          result = 1;
          foreach (var comp in subtreeComplexities) {
            result *= comp + 1;
          }
          break;
        //return subtreeComplexities.Aggregate((a, b) => a == 0 ? b : b == 0 ? a : (a + 1) * (b + 1));
        case OpCodes.Div:
          result = 1;
          foreach (var comp in subtreeComplexities) {
            result *= comp + 1;
          }
          result *= 2;
          break;
        //return subtreeComplexities.Aggregate((a, b) => a == 0 ? b * 2 : b == 0 ? a : (a + 1) * (b + 1) * 2);
        default:
          throw new NotSupportedException();
      }
      return result;
    }
  }
}
